unit fROR_RPLoadSave;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, Buttons, uROR_ListView, ActnList,
  fROR_VistARegistries, Menus;

type
  TFormRPLoadSave = class(TForm)
    cmbbName: TComboBox;
    btnOK: TBitBtn;
    btnCancel: TBitBtn;
    lblName: TLabel;
    lvTemplateNames: TListView;
    alMain: TActionList;
    acOpen: TAction;
    acSave: TAction;
    cmbbDirectory: TComboBox;
    lblDirectory: TLabel;
    spbDelete: TSpeedButton;
    acDelete: TAction;
    mnuPopup: TPopupMenu;
    mnuDelete: TMenuItem;
    acRename: TAction;
    mnuRename: TMenuItem;
    procedure acOpenExecute(Sender: TObject);
    procedure acSaveExecute(Sender: TObject);
    procedure lvTemplateNamesSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure acDeleteExecute(Sender: TObject);
    procedure cmbbDirectorySelect(Sender: TObject);
    procedure alMainUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure lvTemplateNamesKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure acRenameExecute(Sender: TObject);
    procedure lvTemplateNamesEdited(Sender: TObject; Item: TListItem;
      var S: String);
    procedure lvTemplateNamesDblClick(Sender: TObject);

  private
    { fAdmin is True if the user is a registry administrator. }
    fAdmin : Boolean;

    { fInstancePrefix stores the instance name prefix for the report
      parameters templates: 'RegistryIEN:ReportCode:'. }
    fInstancePrefix: String;

    function  getEntity: String;
    function  getInstanceName: String;
    procedure loadParamsList;
    procedure updateNameHistory;

    property InstancePrefix: String read fInstancePrefix;

  public
    function Open(aRegInfo: TRegistryInfo; aPrefix: String): Integer;
    function SaveAs(aRegInfo: TRegistryInfo; aPrefix: String): Integer;

    property Entity: String read getEntity;
    property InstanceName: String read getInstanceName;

  end;

function FormRPLoadSave: TFormRPLoadSave;

implementation
{$R *.dfm}

uses
  uROR_Common, uROR_CommonUtils, uROR_Strings, uROR_Utilities, uROR_Broker,
  fROR_Options;

const
  dirCommon = 0;
  dirUser   = 1;

var
  fFormRPLoadSave: TFormRPLoadSave = nil;

function FormRPLoadSave: TFormRPLoadSave;
begin
  if not Assigned(fFormRPLoadSave) then
    fFormRPLoadSave := TFormRPLoadSave.Create(Application);
  Result := fFormRPLoadSave;
end;

//////////////////////////////// TFormRPLoadSave \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

procedure TFormRPLoadSave.acDeleteExecute(Sender: TObject);
begin
  with lvTemplateNames do
    if Assigned(Selected) and
       (MessageDlg508(rscConfirmation, Format(rscDelete, [Selected.Caption]),
          mtConfirmation, [mbYes,mbNo], 0) = mrYes) then
      begin
        Broker.CreateResults;
        if Broker.CallProc(rpcGUIParameterSet,
          [InstancePrefix+Selected.Caption+U+sReportParamsTemplate, Entity, '@']) then
          Selected.Delete;
      end;
end;

procedure TFormRPLoadSave.acOpenExecute(Sender: TObject);
var
  item: TListItem;
begin
  //--- Check if the template with provided name exists
  item := lvTemplateNames.FindCaption(0, cmbbName.Text, False, True, False);
  if Assigned(item) then
    //--- Make sure that the names match exactly
    cmbbName.Text := item.Caption
  else
    begin
      //--- Display a warning and cancel the action
      MessageDlg508('', Format(rscNotFound, [cmbbName.Text]), mtWarning, [mbOK], 0);
      ModalResult := mrNone;
    end;
  //--- Update the list of template names used during the session
  if ModalResult <> mrNone then
    updateNameHistory;
end;

procedure TFormRPLoadSave.acRenameExecute(Sender: TObject);
begin
  with lvTemplateNames do
    if Assigned(Selected) then
      Selected.EditCaption;
end;

procedure TFormRPLoadSave.acSaveExecute(Sender: TObject);
var
  item: TListItem;
begin
  //--- Check if the template with provided name already exists
  item := lvTemplateNames.FindCaption(0, cmbbName.Text, False, True, False);
  if Assigned(item) then
    begin
      //--- Make sure that the names match exactly
      cmbbName.Text := item.Caption;
      //--- Display a warning and request confirmation
      if (MessageDlg508(rscWarning, Format(rscAlreadyExists, [cmbbName.Text]),
            mtWarning, [mbYes,mbNo], 0) <> mrYes) then
        ModalResult := mrNone;
      end;
  //--- Update the list of template names used during the session
  if ModalResult <> mrNone then
    updateNameHistory;
end;

procedure TFormRPLoadSave.alMainUpdate(Action: TBasicAction;
  var Handled: Boolean);
begin
  //--- Disable default behavior of the Ok and Cancel buttons while
  //    editing the caption to avoid undesirable dialog closure
  if lvTemplateNames.IsEditing then
    begin
      btnCancel.Cancel := False;
      btnOk.Default    := False;
    end
  else
    begin
      btnCancel.Cancel := True;
      btnOk.Default    := True;
    end;

  //--- Disable actions related to the selected template if there is no selection
  if Assigned(lvTemplateNames.Selected) then
    begin
      acDelete.Enabled := True;
      acRename.Enabled := True;
    end
  else
    begin
      acDelete.Enabled := False;
      acRename.Enabled := False;
    end;

  //-- Disable the Open/Save button if a name has not been entered/selected
  if cmbbName.Text <> '' then
    begin
      acOpen.Enabled   := True;
      acSave.Enabled   := True;
    end
  else
    begin
      acOpen.Enabled   := False;
      acSave.Enabled   := False;
    end;
end;

procedure TFormRPLoadSave.cmbbDirectorySelect(Sender: TObject);
var
  ie: TCCRInterfaceElement;
begin
  with lvTemplateNames do
    begin
      //--- Only registry administrators can edit common templates
      ReadOnly := (not fAdmin) and (cmbbDirectory.ItemIndex = dirCommon);
      if ReadOnly then
        begin
          ie := CCROptions.IE[ieReadOnly];
          Color := ie.Color;
          Font.Color := ie.FontColor;
        end
      else
        begin
          Color := clWindow;
          Font.Color := clWindowText;
        end;
    end;
  //--- Reload the list of templates
  loadParamsList;
end;

function TFormRPLoadSave.getEntity: String;
begin
  if cmbbDirectory.ItemIndex = dirCommon then
    Result := 'PKG'
  else
    Result := 'USR';
end;

function TFormRPLoadSave.getInstanceName: String;
begin
  //--- Instance name format: {RegistryIEN}:{ReportCode}:{TemplateName}
  Result := InstancePrefix + cmbbName.Text;
end;

procedure TFormRPLoadSave.loadParamsList;
var
  i, n: Integer;
begin
  Screen.Cursor := crHourGlass;
  try
    Broker.CreateResults;
    //--- Clear the list of template names
    lvTemplateNames.Clear;
    //--- Load the list of template names
    if Broker.CallProc('ROR LIST PARAMETER INSTANCES',
       [sReportParamsTemplate, Entity, InstancePrefix]) then
      begin
        lvTemplateNames.Items.BeginUpdate;
        try
          n := Broker.Results.Count - 1;
          for i:=1 to n do
            lvTemplateNames.AddItem(Piece(Broker.Results[i], U), nil);
        finally
          lvTemplateNames.Items.EndUpdate;
        end;
      end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TFormRPLoadSave.lvTemplateNamesDblClick(Sender: TObject);
begin
  btnOk.Click;
end;

procedure TFormRPLoadSave.lvTemplateNamesEdited(Sender: TObject;
  Item: TListItem; var S: String);
var
  newItem: TListItem;
  fRen: Boolean;
begin
  if S <> Item.Caption then
    begin
      Broker.CreateResults;
      fRen := True;
      //--- Check if a template with the new name already exists
      newItem := lvTemplateNames.FindCaption(0, S, False, True, False);
      if Assigned(newItem) then
        //--- If it exists but it is the same template (only the
        //    case is different) then ignore it
        if newItem = Item then
          newItem := nil
        //--- If a different template has the same name then
        //    display a warning and request confirmation
        else if MessageDlg508(rscWarning,
                  Format(rscAlreadyExists, [cmbbName.Text]),
                  mtWarning, [mbYes,mbNo], 0) <> mrYes then
          fRen := False;

      if fRen then
        //--- Rename the template
        if Broker.CallProc(rpcGUIParameterRename,
             [Entity, sReportParamsTemplate, InstancePrefix+Item.Caption,
             InstancePrefix+S]) then
          begin
            //--- Update the Name feild with the new template name
            cmbbName.Text := S;
            //--- Remove the name of the overwritten template from the list
            if Assigned(newItem) then
              newItem.Delete;
          end
        //--- If the RPC has failed, restore the original template name 
        else
          S := Item.Caption;
    end;
end;

procedure TFormRPLoadSave.lvTemplateNamesKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_Delete) and (Shift = []) then
    acDelete.Execute;
end;

procedure TFormRPLoadSave.lvTemplateNamesSelectItem(Sender: TObject;
  Item: TListItem; Selected: Boolean);
begin
  //--- Update the Name field with the name of selected template
  if Selected then
    cmbbName.Text := Item.Caption;
end;

function TFormRPLoadSave.Open(aRegInfo: TRegistryInfo; aPrefix: String): Integer;
begin
  fAdmin := aRegInfo.Admin;
  fInstancePrefix := aPrefix;

  //--- Customize the dialog box
  Caption := 'Open Report Parameters';
  btnOk.Action := acOpen;
  lblDirectory.Caption := 'Look in:';

  //--- Load the list of template names and display the dialog box
  loadParamsList;
  Result := ShowModal;
end;

function TFormRPLoadSave.SaveAs(aRegInfo: TRegistryInfo; aPrefix: String): Integer;
begin
  fAdmin := aRegInfo.Admin;
  fInstancePrefix := aPrefix;

  //--- Customize the dialog box
  Caption := 'Save Report Parameters As';
  btnOk.Action := acSave;
  lblDirectory.Caption := 'Save in:';

  //--- Only registry administrators can modify common templates
  with cmbbDirectory do
    begin
      Enabled := fAdmin;
      if not Enabled then
        ItemIndex := dirUser;
    end;

  //--- Load the list of template names and display the dialog box
  loadParamsList;
  Result := ShowModal;
end;

procedure TFormRPLoadSave.updateNameHistory;
begin
  with cmbbName do
    begin
      //--- Store the current template name
      Items.Insert(0, Text);
      //--- Limit the history size to 15 items
      if Items.Count > 15 then
        Items.Delete(Items.Count);
    end;
end;

end.
