unit fROR_ReportSelector;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, ovcstate, uROR_State, ovcbase,
  ovcfiler, ovcstore, uROR_VistAStore, ImgList, uROR_Broker, ActnList,
  ComCtrls, uROR_ListView, fROR_ReportParams, uROR_Selector,
  uROR_CustomControls, uROR_ParamsPanel, fROR_VistARegistries, uROR_Errors,
  uROR_Reports, o32editf, o32flxed, uROR_CustomListView,
  VA508AccessibilityManager;

type
  TFormReportSelector = class(TForm)
    pnlButtons: TPanel;
    pnlMain: TPanel;
    btnRun: TBitBtn;
    btnCancel: TBitBtn;
    pnlReportList: TPanel;
    splMain: TSplitter;
    pnlCurrentReport: TPanel;
    pnlReportName: TPanel;
    imlSort: TImageList;
    vxsFormLayout: TCCRVistAStore;
    vxsFormState: TCCRFormState;
    vxsComponentState: TCCRComponentState;
    ActionList: TActionList;
    acShowReportList: TAction;
    lvReports: TCCRListView;
    chkbShowReportList: TCheckBox;
    pnlReportParams: TPanel;
    Panel1: TPanel;
    pnlRightSpacer: TPanel;
    pnlBottomSpacer: TPanel;
    pnlLeftSpacer: TPanel;
    btnSaveParams: TBitBtn;
    btnLoadParams: TBitBtn;
    acLoadParams: TAction;
    acSaveParams: TAction;
    btnDefaultParams: TBitBtn;
    acDefaultParams: TAction;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    procedure vxsFormLayoutBeforeOpen(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure lvReportsSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure vxsFormStateRestoreState(Sender: TObject);
    procedure vxsFormStateSaveState(Sender: TObject);
    procedure acShowReportListExecute(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btnRunClick(Sender: TObject);
    procedure acSelectedUpdate(Sender: TObject);
    procedure acLoadParamsExecute(Sender: TObject);
    procedure acSaveParamsExecute(Sender: TObject);
    procedure lvReportsEnter(Sender: TObject);
    procedure acDefaultParamsExecute(Sender: TObject);

  private
    fParamsFrame: TFrameReportParameters;
    fRegInfo:     TRegistryInfo;

    function getDateTimeToStart: String;
    function getReportDescriptor: TReportDescriptor;
    function getSchedule: String;
    function makeInstancePrefix(aReport: TReportDescriptor): String;

  protected
    function SelectReport(const aReportCode: String): Boolean; virtual;

    property RegInfo: TRegistryInfo              read  fRegInfo;

  public
    function  Load(aRegInfo: TRegistryInfo; Reload: Boolean = False): TReturnCode;
    function  Open(aRegInfo: TRegistryInfo; const aReportCode: String = ''): Boolean;

    property DateTimeToStart: String             read getDateTimeToStart;
    property ParamsFrame: TFrameReportParameters read fParamsFrame;
    property ReportDescriptor: TReportDescriptor read getReportDescriptor;
    property Schedule: String                    read getSchedule;

  published
    procedure ApplyOptions;

  end;

function ReportSelector: TFormReportSelector;

implementation
{$R *.dfm}

uses
  uROR_Common, uROR_CommonUtils, fROR_RPLoadSave, fROR_Options, uROR_Strings;

var
  FormReportSelector: TFormReportSelector = nil;

function ReportSelector: TFormReportSelector;
begin
  if not Assigned(FormReportSelector) then
    FormReportSelector := TFormReportSelector.Create(Application);
  Result := FormReportSelector;
end;

/////////////////////////////// TFormReportSelector \\\\\\\\\\\\\\\\\\\\\\\\\\\\

procedure TFormReportSelector.acDefaultParamsExecute(Sender: TObject);
begin
  with ReportDescriptor do
    begin
      RestoreDefaultParams;
      ParamsFrame.SetFieldValues(Code, Parameters);
    end;
end;

procedure TFormReportSelector.acLoadParamsExecute(Sender: TObject);
var
  buf: TStringList;
begin
  if Assigned(ReportDescriptor) then
    with FormRPLoadSave do
      if Open(RegInfo, makeInstancePrefix(ReportDescriptor)) = mrOk then
        begin
          buf := TStringList.Create;
          try
            if Broker.CallProc(rpcGUIParameterGet,
              [InstanceName+U+sReportParamsTemplate, Entity],
              nil, buf) then
              begin
                buf.Delete(0);
                ReportDescriptor.Parameters.Assign(buf);
                with ReportDescriptor do
                  ParamsFrame.SetFieldValues(Code, Parameters);
              end;
          finally
            buf.Free;
          end;
        end;
end;

procedure TFormReportSelector.acSaveParamsExecute(Sender: TObject);
var
  buf: TStringList;
begin
  if Assigned(ReportDescriptor) then
    with FormRPLoadSave do
      if SaveAs(RegInfo, makeInstancePrefix(ReportDescriptor)) = mrOk then
        begin
          Broker.CreateResults;
          buf := TStringList.Create;
          try
            with ReportDescriptor do
              ParamsFrame.GetFieldValues(Code, Parameters, gfmSave);
            ReportDescriptor.Parameters.AssignTo(buf);
            Broker.CallProc(rpcGUIParameterSet,
              [InstanceName+U+sReportParamsTemplate, Entity], buf);
          finally
            buf.Free;
          end;
        end;
end;

procedure TFormReportSelector.acSelectedUpdate(Sender: TObject);
begin
  TCustomAction(Sender).Enabled := Assigned(lvReports.Selected);
end;

procedure TFormReportSelector.acShowReportListExecute(Sender: TObject);
begin
  with chkbShowReportList do
    if Checked then
      begin
        pnlReportList.Visible := True;
        splMain.Visible       := True;
      end
    else
      begin
        splMain.Visible       := False;
        pnlReportList.Visible := False;
      end;
end;

procedure TFormReportSelector.ApplyOptions;
var
  ie: TCCRInterfaceElement;
begin
  ie := CCROptions.IE[ieReadOnly];
  lvReports.Color := ie.Color;
  lvReports.Font.Color := ie.FontColor;
end;

procedure TFormReportSelector.btnRunClick(Sender: TObject);
var
  errCtrl: TWinControl;
begin
  ModalResult := mrNone;
  if Assigned(lvReports.Selected) then
    begin
      if Assigned(ReportDescriptor) then
        begin
          with ReportDescriptor do
            errCtrl := ParamsFrame.GetFieldValues(Code, Parameters, gfmRun);
          if Assigned(errCtrl) then
            try
              ActiveControl := errCtrl;
            except
            end
          else
            ModalResult := mrOk;
        end;
    end;
end;

procedure TFormReportSelector.FormCreate(Sender: TObject);
begin
  ApplyOptions;
  //--- Create the report parameters frame
  fParamsFrame := TFrameReportParameters.Create(Self);
  fParamsFrame.Align := alClient;
  fParamsFrame.Parent := pnlReportParams;
end;

procedure TFormReportSelector.FormShow(Sender: TObject);
begin
  //--- Make sure that parameter panels are configured properly
  with lvReports do
    if Assigned(Selected) then
      begin
        //--- Force panel setup for the current report
        ItemFocused := Selected;
        Selected    := nil;
        Selected    := ItemFocused;
      end
    else if Items.Count > 0 then
      begin
        //--- By default, select the first available report from the list
        Selected    := Items[0];
        ItemFocused := Selected;
      end;
end;

function TFormReportSelector.getDateTimeToStart: String;
begin
  Result := FloatToStr(WindowsDateTimeToFMDateTime(ParamsFrame.GetDTRun));
end;

function TFormReportSelector.getReportDescriptor: TReportDescriptor;
begin
  if Assigned(lvReports.Selected) then
    Result := lvReports.Selected.Data
  else
    Result := nil;
end;

function TFormReportSelector.getSchedule: String;
begin
  Result := ParamsFrame.cmbbSchedule.Text;
end;

function TFormReportSelector.Load(aRegInfo: TRegistryInfo; Reload: Boolean): TReturnCode;
var
  i, n:Integer;
begin
  Result := RC_OK;
  ParamsFrame.RegInfo := aRegInfo;
  if (aRegInfo <> fRegInfo) or Reload then
    begin
      ParamsFrame.UpdatePanelReferences;
      fRegInfo := aRegInfo;
      n := RegInfo.AvailableReports.Count - 1;
      // Populate the list of reports
      lvReports.Clear;
      for i:=0 to n do
        lvReports.AddItem(RegInfo.AvailableReports[i].Name, RegInfo.AvailableReports[i]);
      for i:=lvReports.Items.Count-1 downto 0 do
          lvReports.Items[i].Indent := 0;
      // Hide all parameter panels
      ParamsFrame.SetupPanels(nil);
    end;
end;

procedure TFormReportSelector.lvReportsEnter(Sender: TObject);
begin
  with lvReports do
    if (Selected = nil) and (ItemFocused <> nil) then
      Selected := ItemFocused;
end;

procedure TFormReportSelector.lvReportsSelectItem(Sender: TObject;
  Item: TListItem; Selected: Boolean);
var
  rpt: TReportDescriptor;
begin
  if not (csDestroying in ComponentState) then
    begin
      rpt := Item.Data;
      if Selected then
        begin
          pnlReportName.Caption := '  ' + rpt.Name;
          Item.ImageIndex := 2;
          ParamsFrame.SetupPanels(rpt);
        end
      else
        begin
          pnlReportName.Caption := '';
          Item.ImageIndex := -1;
          if Assigned(rpt) then
            ParamsFrame.GetFieldValues(rpt.Code, rpt.Parameters);
          ParamsFrame.SetupPanels(nil);
        end;
    end;
end;

function TFormReportSelector.makeInstancePrefix(aReport: TReportDescriptor): String;
begin
  if aReport.SharedTemplates then
    Result := aReport.Code + '::'
  else
    Result := aReport.Code + ':' + RegInfo.Name + ':';
end;

function TFormReportSelector.Open(aRegInfo: TRegistryInfo;
  const aReportCode: String): Boolean;
begin
  Load(aRegInfo);
  if aReportCode <> '' then
    begin
      chkbShowReportList.Checked := False;
      chkbShowReportList.Enabled := False;
      SelectReport(aReportCode);
      Caption := aRegInfo.Description + ' ' + RSC00240;
    end
  else
    begin
      chkbShowReportList.Checked := True;
      chkbShowReportList.Enabled := True;
      Caption := aRegInfo.Description + ' ' + RSC00241;
    end;
  Result := (ShowModal = mrOk);
end;

function TFormReportSelector.SelectReport(const aReportCode: String): Boolean;
var
  i, n:Integer;
begin
  Result := False;
  with lvReports do
    begin
      n := Items.Count - 1;
      for i:=0 to n do
        if TReportDescriptor(Items[i].Data).Code = aReportCode then
          begin
            Selected := Items[i];
            Result := True;
            Break;
          end;
    end;
end;

procedure TFormReportSelector.vxsFormLayoutBeforeOpen(Sender: TObject);
begin
  vxsFormLayout.Broker := Broker;
end;
{
  The Active property of the vxsComponentState component is set to False so
  that it will not save/restore the component properies automatically.  The
  corresponding save/restore methods are called from the event handlers of
  the vxsFormState component to minimize network data exchange with VistA
  (the VistA storage is opened and closed only once per save/restore process).
}
procedure TFormReportSelector.vxsFormStateRestoreState(Sender: TObject);
begin
  vxsComponentState.RestoreState;
  if Assigned(ParamsFrame) then
    ParamsFrame.vxsComponentState.RestoreState;
end;

procedure TFormReportSelector.vxsFormStateSaveState(Sender: TObject);
begin
  vxsComponentState.SaveState;
  if Assigned(ParamsFrame) then
    ParamsFrame.vxsComponentState.SaveState;
end;

////////////////////////////////////////////////////////////////////////////////

initialization
finalization
  FormReportSelector := nil;

end.
