unit uROR_Logs;

interface

uses
  SysUtils, Classes, Graphics, HTMLLite, ComCtrls;

type
  TCCRImageType = (itDefault, itSelected, itExpanded);

  //------------------------------ TCCRHTMLParams ------------------------------

  TCCRHTMLParams = class(TPersistent)
  private
    fBgColor: String;
    fColor:   String;

  public
    constructor Create(anHTMLView: THtmlLite);

    property BgColor: String read fBgColor;
    property Color: String   read fColor;

  end;

  //----------------------------- TCCRActivityNode -----------------------------

  TCCRActivityNode = class(TPersistent)
  public
    function Expand(Sender: TTreeView; Node: TTreeNode): Boolean; virtual;
    function Get508Details: String; virtual;
    function GetHTML(Params: TCCRHTMLParams): String; virtual;
    function GetImageIndex(const anImageType: TCCRImageType = itDefault): Integer; virtual;

  end;

  //-------------------------------- TCCRAVNode --------------------------------

  TCCRAVNode = class(TCCRActivityNode)
  private
    fList: TStringList;

  public
    constructor Create;
    destructor  Destroy; override;

    function Get508Details: String; override;
    function GetHTML(Params: TCCRHTMLParams): String; override;
    function GetImageIndex(const anImageType: TCCRImageType = itDefault): Integer; override;

    property List: TStringList read fList;

  end;

  //------------------------------- TCCRLogNode --------------------------------

  TCCRLogNode = class(TCCRActivityNode)
  private
    fIEN:    String;
    fDescr:  String;
    fRegLst: TStringList;

  public
    constructor Create(const RawData: String);
    destructor  Destroy; override;

    function Get508Details: String; override;
    function GetHTML(Params: TCCRHTMLParams): String; override;
    function GetImageIndex(const anImageType: TCCRImageType = itDefault): Integer; override;

    property IEN: String         read fIEN    write fIEN;
    property Descr: String       read fDescr  write fDescr;
    property RegLst: TStringList read fRegLst;

  end;

  //------------------------------ TCCRMessageNode -----------------------------

  TCCRMessageNode = class(TCCRActivityNode)
  private
    fDetails: TStringList;
    fIEN:     String;
    fMsg:     String;
    fDescr:   String;

  public
    constructor Create(const RawData: String);
    destructor  Destroy; override;

    function Get508Details: String; override;
    function GetHTML(Params: TCCRHTMLParams): String; override;
    function GetImageIndex(const anImageType: TCCRImageType = itDefault): Integer; override;

    property IEN: String          read fIEN     write fIEN;
    property Details: TStringList read fDetails;
    property Msg: String          read fMsg     write fMsg;
    property Descr: String        read fDescr   write fDescr;

  end;

implementation

uses
  uROR_Common, uROR_Utilities;

const
  ENDOFLOG = 'L^END';

//***** Converts a color to an html color string (e.g. clRed => #FF0000)

function ColorToHtml(const Color: TColor): String;
var
  iRgb: Longint;
  iHtml: Longint;
begin
  //--- Convert system colors to rgb colors
  iRgb := ColorToRGB(Color);
  //--- Change BBGGRR to RRGGBB
  iHtml := ((iRgb and $0000FF) shl 16) or // shift red to the left
           ( iRgb and $00FF00) or         // green keeps the place
           ((iRgb and $FF0000) shr 16);   // shift blue to the right
  //--- Create the html string
  Result := '#' + IntToHex(iHtml, 6);
end;

///////////////////////////////// TCCRHTMLParams \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

constructor TCCRHTMLParams.Create(anHTMLView: ThtmlLite);
begin
  inherited Create;
  fBgColor := ColorToHtml(anHTMLView.DefBackground);
  fColor   := ColorToHtml(anHTMLView.DefFontColor);
end;

//////////////////////////////// TCCRActivityNode \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

function TCCRActivityNode.Expand(Sender: TTreeView; Node: TTreeNode): Boolean;
begin
  Result := False;
end;

function TCCRActivityNode.Get508Details: String;
begin
  Result := '';
end;

function TCCRActivityNode.GetHTML(Params: TCCRHTMLParams): String;
begin
  Result := '';
end;

function TCCRActivityNode.GetImageIndex(const anImageType: TCCRImageType): Integer;
begin
  Result := -1;
end;

/////////////////////////////////// TCCRAVNode \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

constructor TCCRAVNode.Create;
begin
  inherited Create;
  fList := TStringList.Create;
end;

destructor TCCRAVNode.Destroy;
begin
  FreeAndNil(fList);
  inherited;
end;

function TCCRAVNode.Get508Details: String;
var
  i: Integer;
  dt, tmp: String;

  function tcv(const aName: String; aValue: String): String;
  begin
    if aValue = '' then
      aValue := 'blank';
    Result := aName + aValue;
  end;
begin
  Result := 'Access Violations' + U;
  for i := 0 to List.Count - 1 do
  begin
    tmp := List.Strings[i];
    dt := FMDateTimeStr(Piece(tmp,U), [fmdtShortDate,fmdtTime24]);
    Result := Result + tcv('Date/Time: ',dt)
      + tcv('User Name: ', Piece(tmp,U,2))
      + tcv('User IEN: ', Piece(tmp,U,3))
      + tcv('Message: ', Piece(tmp,U,4)) + U;
  end;
end;

function TCCRAVNode.GetHTML(Params: TCCRHTMLParams): String;
var
  i, n: Integer;
  dt, tmp: String;

  function tcv(aValue: String): String;
  begin
    if aValue = '' then
      aValue := '&nbsp;';
    with Params do
        Result := '<td bgcolor="' + BgColor + '">' + aValue + '</td>';
  end;

begin
  Result :=
    '<html><body><p><strong>Access Violations</strong></p>' +
    '<table width = "100%" border="0" cellspacing="0" cellpadding="0" bgcolor="' +
    Params.Color + '"><tr><td>' +
    '<table width = "100%" border="0" cellspacing="1" cellpadding="5">' +
    '<tr>' + tcv('<b>Date/Time</b>') + tcv('<b>User Name</b>') +
    tcv('<b>User IEN</b>') + tcv('<b>Message</b>') +'</th></tr>';

  n := List.Count - 1;
  for i:=0 to n do
    begin
      tmp := List.Strings[i];
      dt := FMDateTimeStr(Piece(tmp,U), [fmdtShortDate,fmdtTime24]);
      Result := Result + '<tr valign="top">' + tcv(dt) + tcv(Piece(tmp,U,2)) +
        tcv(Piece(tmp,U,3)) + tcv(Piece(tmp,U,4)) + '</tr>';
    end;

  Result := Result + '</table></td></tr></table></body></html>';
end;

function TCCRAVNode.GetImageIndex(const anImageType: TCCRImageType): Integer;
begin
  if anImageType = itSelected then
    Result := 10
  else
    Result := 9;
end;

////////////////////////////////// TCCRLogNode \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

constructor TCCRLogNode.Create(const RawData: String);
begin
  inherited Create;
  fDescr  := RawData;
  fIEN    := Piece(RawData, U, 2);
  fRegLst := TStringList.Create;
end;

destructor TCCRLogNode.Destroy;
begin
  FreeAndNil(fRegLst);
  inherited;
end;

function TCCRLogNode.Get508Details: String;
var
  rls, title: String;
  i: Integer;

  function tcv(const aName: String; aValue: String): String;
  begin
    if aValue = '' then
      aValue := 'blank';
    Result := aName + aValue + U;
  end;
begin
  title := Piece(Descr, U, 3) + U;
  if title = U then
    title := 'No title' + U;
  rls := '';
  for i := 0 to RegLst.Count - 1 do
    rls := rls + ', ' + Piece(RegLst.Strings[i], U, 3);
  Result := title +
    tcv('Start Date/Time: ', Piece(Descr,U,5)) +
    tcv('End Date/Time: ', Piece(Descr,U,8)) +
    tcv('Job Number: ', Piece(Descr,U,7)) +
    tcv('Task Number: ', Piece(Descr,U,12)) +
    tcv('Number of Processed Patients: ', Piece(Descr,U,9)) +
    tcv('Number of Patients Processed with Errors: ', Piece(Descr,U,10)) +
    tcv('Processing Rate (pt/sec): ', Piece(Descr,U,11)) +
    tcv('Affected Registries: ', Piece(rls, ', ', 2, 99));
end;

function TCCRLogNode.GetHTML(Params: TCCRHTMLParams): String;
var
  i, n: Integer;
  rls, title: String;

  function tcv(const aName: String; aValue: String; firstRow: Boolean = False): String;
  begin
    if aValue = '' then
      aValue := '&nbsp;';
    with Params do
      if firstRow then
        Result := '<tr><td width="210" bgcolor="' + BgColor + '">' + aName +
          ':</td><td bgcolor="' + BgColor + '">' + aValue +
          '</td></tr>'
      else
        Result := '<tr><td bgcolor="' + BgColor + '">' + aName +
          ':</td><td bgcolor="' + BgColor + '">' + aValue + '</td></tr>';
  end;

begin
  title := Piece(Descr, U, 3);
  if title = '' then
    title := '&nbsp;';

  n := RegLst.Count - 1;
  for i:=0 to n do
    rls := rls + ', ' + Piece(RegLst.Strings[i], U, 3);

  Result :=
    '<html><body><p><strong>' + title + '</strong></p>' +
    '<table width = "100%" border="0" cellspacing="0" cellpadding="0" bgcolor="' +
    Params.Color + '"><tr><td>' +
    '<table width = "100%" border="0" cellspacing="1" cellpadding="5">' +
    tcv('Start Date/Time', Piece(Descr,U,5), True) +
    tcv('End Date/Time', Piece(Descr,U,8)) +
    tcv('Job Number', Piece(Descr,U,7)) +
    tcv('Task Number', Piece(Descr,U,12)) +
    tcv('Number of Processed Patients', Piece(Descr,U,9)) +
    tcv('Number of Patients Processed with Errors', Piece(Descr,U,10)) +
    tcv('Processing Rate (pt/sec)', Piece(Descr,U,11)) +
    tcv('Affected Registries', Piece(rls, ', ', 2, 999)) +
    '</table></td></tr></table></body></html>';
end;

function TCCRLogNode.GetImageIndex(const anImageType: TCCRImageType): Integer;
begin
  if anImageType = itExpanded then
    Result := 1
  else
    Result := 0;
end;

//////////////////////////////// TCCRMessageNode \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

constructor TCCRMessageNode.Create(const RawData: String);
begin
  inherited Create;
  fDescr   := RawData;
  fDetails := TStringList.Create;
  fIEN     := Piece(RawData, U, 2);
end;

destructor TCCRMessageNode.Destroy;
begin
  FreeAndNil(fDetails);
  inherited;
end;

function TCCRMessageNode.Get508Details: String;
var
  ils, title: String;
  i: Integer;

  function tcv(const aName: String; aValue: String): String;
  begin
    if aValue = '' then
      aValue := 'blank';
    Result := aName + aValue + U;
  end;
begin
  if Msg <> '' then
    title := Msg + U
  else
    title := 'No title' + U;
  ils := '';
  for i := 0 to Details.Count-1 do
    ils := ils + chr(13) + Piece(Details.Strings[i], U, 2, 999);
  ils := Piece(ils, chr(13), 2, 999);
  Result := title +
    tcv('Date/Time: ', Piece(Descr,U,5)) +
    tcv('Type: ', Piece(Descr,U,3)) +
    tcv('Patient Name: ', Piece(Descr,U,7)) +
    tcv('Patient IEN (DFN): ', Piece(Descr,U,8)) +
    tcv('Additional Information: ', ils);
end;

function TCCRMessageNode.GetHTML(Params: TCCRHTMLParams): String;
var
  ils, title: String;
  i, n: Integer;

  function tcv(const aName: String; aValue: String; firstRow: Boolean = False): String;
  begin
    if aValue = '' then
      aValue := '&nbsp;';
    with Params do
      if firstRow then
        Result := '<tr valign="top"><td width="210" bgcolor="' + BgColor + '">' +
          aName + ':</td><td bgcolor="' + BgColor + '">' + aValue + '</td></tr>'
      else
        Result := '<tr valign="top"><td bgcolor="' + BgColor + '">' + aName +
          ':</td><td bgcolor="' + BgColor + '">' + aValue + '</td></tr>';
  end;

begin
  if Msg <> '' then
    title := Msg
  else
    title := '&nbsp;';

  ils := '';
  n := Details.Count - 1;
  for i:=0 to n do
    ils := ils + '<br>' + Piece(Details.Strings[i], U, 2, 999);
  ils := Piece(ils, '<br>', 2, 999);
  if ils <> '' then
    ils := '<pre><font size="+1">' + ils + '</font></pre>';

  Result :=
    '<html><body><p><strong>' + title + '</strong></p>' +
    '<table width = "100%" border="0" cellspacing="0" cellpadding="0" bgcolor="' +
    Params.Color + '"><tr><td>' +
    '<table width = "100%" border="0" cellspacing="1" cellpadding="5">' +
    tcv('Date/Time', Piece(Descr,U,5), True) +
    tcv('Type', Piece(Descr,U,3)) +
    tcv('Patient Name', Piece(Descr,U,7)) +
    tcv('Patient IEN (DFN)', Piece(Descr,U,8)) +
    tcv('Additional Information', ils) +
    '</table></td></tr></table></body></html>';
end;

function TCCRMessageNode.GetImageIndex(const anImageType: TCCRImageType): Integer;
begin
  case StrToIntDef(Piece(Descr,U,4), 0) of
    2:   Result := 6;  // Information
    3:   Result := 8;  // Data Quality
    4:   Result := 5;  // Warning
    5:   Result := 7;  // Database Error
    6:   Result := 4;  // Error
    else Result := -1;
  end;
end;

end.
