unit uROR_PatientDataLists;

interface

uses
  Classes, uROR_Common, uROR_Utilities, uROR_Broker, uROR_Errors, SysUtils;

type

  //------------------------------- TLabGroupStat(s) ---------------------------

  TLabGroupStat = class(TCollectionItem)
  private
    fGroupCode:   String;
    fGroupName:   String;
    fLatestValue: String;
    fLatestDate:  String;
    fLowestValue: String;
    fLowestDate:  String;

    function  getRawData: String;
    procedure setGroupCode(const aValue: String);
    procedure setRawData(const RawData: String);

  public
    constructor Create(aCollection: TCollection); override;

    property GroupCode: String   read  fGroupCode;
    property GroupName: String   read  fGroupName;
    property LatestValue: String read  fLatestValue;
    property LatestDate: String  read  fLatestDate;
    property LowestValue: String read  fLowestValue;
    property LowestDate: String  read  fLowestDate;
    property RawData: String     read  getRawData
                                 write setRawData;

  end;

  TLabGroupStats = class(TCollection)
  private
    fIndex: TStringList;

  protected
    function GetItem(anIndex: Integer): TLabGroupStat;

  public
    constructor Create;
    destructor  Destroy; override;

    function  Add(const RawData: String): TLabGroupStat;
    procedure AssignRawData(RawData: TStrings);
    function  Find(const aGroupCode: String): TLabGroupStat; overload;
    function  Find(const aGroupCode: TLabGroupCode): TLabGroupStat; overload;
    procedure Notify(Item: TCollectionItem; Action: TCollectionNotification); override;

    property Items[Index: Integer]: TLabGroupStat read GetItem; default;

  end;

  //-------------------------------- TLabResult(s) -----------------------------

  TLabResult = class(TCollectionItem)
  private
    fDate:      String;
    fGroupCode: String;
    fGroupName: String;
    fResultIEN: String;
    fTestIEN:   String;
    fTestName:  String;
    fValue:     String;

    function  getRawData: String;
    procedure setRawData(const RawData: String);

  public
    constructor Create(aCollection: TCollection); override;

    property Date: String        read  fDate;
    property GroupCode: String   read  fGroupCode;
    property GroupName: String   read  fGroupName;

    property RawData: String     read  getRawData
                                 write setRawData;

    property ResultIEN: String   read  fResultIEN;
    property TestIEN: String     read  fTestIEN;
    property TestName: String    read  fTestName;
    property Value: String       read  fValue;
  end;

  TLabResults = class(TCollection)
  private
    fStats: TLabGroupStats;

  protected
    function GetItem(anIndex: Integer): TLabResult;

  public
    constructor Create;
    destructor  Destroy; override;

    function Add(const RawData: String): TLabResult;
    procedure AssignRawData(RawData: TStrings);
    function  Load(const aRegistryIEN, aPatientIEN: String): TReturnCode;

    property Items[Index: Integer]: TLabResult read GetItem; default;
    property Stats: TLabGroupStats read fStats;

  end;

  //------------------------------- TLocalField(s) -----------------------------

  TLocalField = class(TCollectionItem)
  private
    fComment:     String;
    fDate:        String;
    fDescription: String;
    fFieldIEN:    String;
    fIEN:         String;
    fModified:    Boolean;
    fName:        String;
    fToggle:      Boolean;

    procedure setComment(const aValue: String);
    procedure setDate(const aValue: String);
    procedure setFieldIEN(const aValue: String);
    procedure setIEN(const aValue: String);
    procedure setModified(const aValue: Boolean);
    procedure setToggle(const aValue: Boolean);

  protected
    function  GetRawData: String; virtual;
    procedure SetRawData(const aData: String); virtual;

  public
    constructor Create(aCollection: TCollection); override;

    procedure AssignRawDefinition(RawData: String);

    property Comment: String                    read  fComment
                                                write setComment;

    property Date: String                       read  fDate
                                                write setDate;

    property Description: String                read  fDescription;

    property FieldIEN: String                   read  fFieldIEN;

    property IEN: String                        read  fIEN
                                                write setIEN;

    property Modified: Boolean                  read  fModified
                                                write setModified;

    property Name: String                       read  fName;

    property RawData: String                    read  GetRawData
                                                write SetRawData;

    property Toggle: Boolean                    read  fToggle
                                                write setToggle;

  end;

  TLocalFields = class(TCollection)
  private
    fIndex:    TStringList;
    fModified: Boolean;
    fRegIEN:   String;

    function  reloadDefinitions: TReturnCode;
    procedure setModified(const aValue: Boolean);

  protected
    function  AddDefinition(const RawData: String): TLocalField; virtual;
    function  GetItem(anIndex: Integer): TLocalField;
    procedure Notify(Item: TCollectionItem; Action: TCollectionNotification); override;
    procedure Update(Item: TCollectionItem); override;

  public
    constructor Create;
    destructor  Destroy; override;

    procedure AssignRawData(PtData: TStrings);
    function  Find(const anIEN: String): TLocalField;
    function  LoadDefinitions(const aRegistryIEN: String): TReturnCode;
    procedure PrepareRawData(PtData: TStrings);

    property Items[Index: Integer]: TLocalField read GetItem; default;

    property Modified: Boolean                  read  fModified
                                                write setModified;

    property RegIEN: String read fRegIEN;

  end;

  //-------------------------------- TSetOfCodes -------------------------------

  TCodeSetItem = class(TCollectionItem)
  private
    fCode: String;
    fText: String;

  public
    procedure AssignRawData(RawData: String);

    property Code: String                       read fCode;
    property Text: String                       read fText;

  end;

  TSetOfCodes = class(TCollection)
  private
    fType: String;

  protected
    function GetItem(anIndex: Integer): TCodeSetItem;

  public
    constructor Create(const aType: String);

    function  Add(RawData: String): TCodeSetItem;
    procedure AssignTo(aDest: TPersistent); override;
    function  IndexOfCode(aCode: String): Integer;
    function  Load(const aRegistryIEN: String): TReturnCode;

    property Items[Index: Integer]: TCodeSetItem read GetItem; default;

  end;

  //------------------------------ TSelectionRule(s) ---------------------------

  TSelectionRule = class(TCollectionItem)
  private
    fDate: String;
    fDescription: String;
    fLocation: String;
    fName: String;

  public
    procedure AssignRawData(RawData: String);

    property Date: String                       read fDate;
    property Description: String                read fDescription;
    property Location: String                   read fLocation;
    property Name: String                       read fName;

  end;

  TSelectionRules = class(TCollection)
  protected
    function GetItem(anIndex: Integer): TSelectionRule;

  public
    constructor Create;

    function  Add(const RawData: String): TSelectionRule;
    procedure AssignRawData(PtData: TStrings);
    function  Load(const aRegistryIEN, aPatientIEN: String): TReturnCode;

    property Items[Index: Integer]: TSelectionRule read GetItem; default;

  end;

implementation

///////////////////////////////// TLabGroupStat \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

constructor TLabGroupStat.Create(aCollection: TCollection);
begin
  inherited;
  RawData := '';
end;

function TLabGroupStat.getrawData: String;
begin
  Result := 'LTG' + U + GroupCode + U + GroupName + U +
    LatestValue + U + LatestDate + U +
    LowestValue + U + LowestDate;
end;

procedure TLabGroupStat.setGroupCode(const aValue: String);
var
  i: Integer;
begin
  if aValue <> fGroupCode then
    begin
      if Assigned(Collection) then
        with TLabGroupStats(Collection) do
          begin
            if (fGroupCode <> '') and fIndex.Find(fGroupCode, i) then
              fIndex.Delete(i);
            if aValue <> '' then
              fIndex.AddObject(aValue, Self);
          end;
      fGroupCode := aValue;
    end;
end;

procedure TLabGroupStat.setRawData(const RawData: String);
begin
  setGroupCode(Piece(RawData,U,2));
  fGroupName   := Piece(RawData, U, 3);
  fLatestValue := Piece(RawData, U, 4);
  fLatestDate  := Piece(RawData, U, 5);
  fLowestValue := Piece(RawData, U, 6);
  fLowestDate  := Piece(RawData, U, 7);
end;

///////////////////////////////// TLabGroupStats \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

constructor TLabGroupStats.Create;
begin
  inherited Create(TLabGroupStat);
  fIndex := TStringList.Create;
end;

destructor TLabGroupStats.Destroy;
begin
  FreeAndNil(fIndex);
  inherited;
end;

function TLabGroupStats.Add(const RawData: String): TLabGroupStat;
begin
  Result := TLabGroupStat(inherited Add);
  if Assigned(Result) then
    Result.RawData := RawData;
end;

procedure TLabGroupStats.AssignRawData(RawData: TStrings);
var
  i, i1, n: Integer;
begin
  Clear;
  n := RawData.Count - 1;
  i1 := n + 1;
  for i:=0 to n do
    if Piece(RawData[i], U) = 'LTG' then
      begin
        i1 := i;
        Break;
      end;
  for i:=i1 to n do
    if Piece(RawData[i], U) = 'LTG' then
      Add(RawData[i])
    else
      Break;
end;

function TLabGroupStats.Find(const aGroupCode: String): TLabGroupStat;
var
  i: Integer;
begin
  if fIndex.Find(aGroupCode, i) then
    Result := TLabGroupStat(fIndex.Objects[i])
  else
    Result := nil;
end;

function TLabGroupStats.Find(const aGroupCode: TLabGroupCode): TLabGroupStat;
var
  i: Integer;
begin
  if fIndex.Find(IntToStr(Ord(aGroupCode)), i) then
    Result := TLabGroupStat(fIndex.Objects[i])
  else
    Result := nil;
end;

function TLabGroupStats.GetItem(anIndex: Integer): TLabGroupStat;
begin
  Result := TLabGroupStat(inherited GetItem(anIndex));
end;

procedure TLabGroupStats.Notify(Item: TCollectionItem; Action: TCollectionNotification);
var
  i: Integer;
begin
  inherited;
  if Assigned(fIndex) then
    if (Action = cnDeleting) or (Action = cnExtracting) then
      if fIndex.Find(TLabGroupStat(Item).GroupCode, i) then
        fIndex.Delete(i);
end;

/////////////////////////////////// TLabResult \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

constructor TLabResult.Create(aCollection: TCollection);
begin
  inherited;
  RawData := '';
end;

function TLabResult.getrawData: String;
begin
  Result := 'LTR' + U + ResultIEN + U + Date + U + Value + U +
    TestIEN + U + TestName + U +
    GroupCode + U + GroupName;
end;

procedure TLabResult.setRawData(const RawData: String);
begin
  fResultIEN := Piece(RawData, U, 2);
  fDate      := Piece(RawData, U, 3);
  fValue     := Piece(RawData, U, 4);
  fTestIEN   := Piece(RawData, U, 5);
  fTestName  := Piece(RawData, U, 6);
  fGroupCode := Piece(RawData, U, 7);
  fGroupName := Piece(RawData, U, 8);
end;

////////////////////////////////// TLabResults \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

constructor TLabResults.Create;
begin
  inherited Create(TLabResult);
  fStats := TLabGroupStats.Create;
end;

destructor TLabResults.Destroy;
begin
  FreeAndNil(fStats);
  inherited;
end;

function TLabResults.Add(const RawData: String): TLabResult;
begin
  Result := TLabResult(inherited Add);
  if Assigned(Result) then
    Result.RawData := RawData;
end;

procedure TLabResults.AssignRawData(RawData: TStrings);
var
  i, i1, n: Integer;
begin
  Clear;
  n := RawData.Count - 1;
  i1 := n + 1;
  for i:=0 to n do
    if Piece(RawData[i], U) = 'LTR' then
      begin
        i1 := i;
        Break;
      end;
  for i:=i1 to n do
    if Piece(RawData[i], U) = 'LTR' then
      Add(RawData[i])
    else
      Break;
  Stats.AssignRawData(RawData);
end;

function TLabResults.GetItem(anIndex: Integer): TLabResult;
begin
  Result := TLabResult(inherited GetItem(anIndex));
end;

function TLabResults.Load(const aRegistryIEN, aPatientIEN: String): TReturnCode;
var
  rawData: TStringList;
begin
  Clear;
  rawData := TStringList.Create;
  try
    if Broker.CallProc('ROR PATIENT REGISTRY LABS',
      [aRegistryIEN, aPatientIEN], nil, rawData) then
      begin
        AssignRawData(rawData);
        Result := RC_OK;
      end
    else
      Result := RCE_RPC_ERROR;
  finally
    rawData.Free;
  end;
end;

///////////////////////////////// TLocalField \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

constructor TLocalField.Create(aCollection: TCollection);
begin
  inherited;
  fModified := True; // To avoid unnecessary 'Changed' events
  AssignRawDefinition('');
  RawData := '';
  fModified := False;
end;

procedure TLocalField.AssignRawDefinition(RawData: String);
begin
  setFieldIEN(Piece(RawData, U));
  fName        := Piece(RawData, U, 2);
  fDescription := Piece(RawData, U, 4);
end;

function TLocalField.GetRawData: String;
begin
  Result := 'LFV' + U + IEN + U + FieldIEN + U + Name + U + Date + U + Comment;
end;

procedure TLocalField.setComment(const aValue: String);
begin
  if aValue <> fComment then
    begin
      fComment := aValue;
      Modified := True;
    end;
end;

procedure TLocalField.setDate(const aValue: String);
begin
  if aValue <> fDate then
    begin
      fDate := aValue;
      Modified := True;
    end;
end;

procedure TLocalField.setFieldIEN(const aValue: String);
var
  i: Integer;
begin
  if aValue <> fFieldIEN then
    begin
      if Assigned(Collection) then
        with TLocalFields(Collection) do
          begin
            if (fFieldIEN <> '') and fIndex.Find(fFieldIEN, i) then
              fIndex.Delete(i);
            if aValue <> '' then
              fIndex.AddObject(aValue, Self);
          end;
      fFieldIEN := aValue;
      Modified := True;
    end;
end;

procedure TLocalField.setIEN(const aValue: String);
begin
  if aValue <> fIEN then
    begin
      fIEN := aValue;
      Modified := True;
    end;
end;

procedure TLocalField.setModified(const aValue: Boolean);
begin
  if aValue <> fModified then
    begin
      fModified := aValue;
      if fModified then
        Changed(False);
    end;
end;

procedure TLocalField.SetRawData(const aData: String);
begin
  IEN     := Piece(aData, U, 2);
  Date    := Piece(aData, U, 5);
  Comment := Piece(aData, U, 6);
  Toggle  := (aData <> '');
end;

procedure TLocalField.setToggle(const aValue: Boolean);
begin
  if aValue <> fToggle then
    begin
      fToggle := aValue;
      Modified := True;
    end;
end;

///////////////////////////////// TLocalFields \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

constructor TLocalFields.Create;
begin
  inherited Create(TLocalField);
  fIndex := TStringList.Create;
  fIndex.Sorted := True;
  fIndex.Duplicates := dupError;
  fModified := False;
  fRegIEN := '';
end;

destructor TLocalFields.Destroy;
begin
  FreeAndNil(fIndex);
  inherited;
end;

function TLocalFields.AddDefinition(const RawData: String): TLocalField;
begin
  Result := TLocalField(inherited Add);
  if Assigned(Result) then
    Result.AssignRawDefinition(RawData);
end;

procedure TLocalFields.AssignRawData(PtData: TStrings);
var
  i, i1, n: Integer;
  lf: TLocalField;
begin
  if Count <= 0 then
    reloadDefinitions;

  Modified := False;

  if Count > 0 then
    begin
      n := PtData.Count - 1;
      i1 := n + 1;
      for i:=0 to n do
        if Piece(PtData[i], U) = 'LFV' then
          begin
            i1 := i;
            Break;
          end;
      for i:=i1 to n do
        if Piece(PtData[i], U) = 'LFV' then
          begin
            lf := Find(Piece(PtData[i],U,3));
            if Assigned(lf) then
              lf.RawData := PtData[i];
          end
        else
          Break;
    end;
end;

function TLocalFields.Find(const anIEN: String): TLocalField;
var
  i: Integer;
begin
  if fIndex.Find(anIEN, i) then
    Result := TLocalField(fIndex.Objects[i])
  else
    Result := nil;
end;

function TLocalFields.GetItem(anIndex: Integer): TLocalField;
begin
  Result := TLocalField(inherited GetItem(anIndex));
end;

function TLocalFields.LoadDefinitions(const aRegistryIEN: String): TReturnCode;
begin
  Result := RC_OK;
  if aRegistryIEN <> RegIEN then
    begin
      fRegIEN := aRegistryIEN;
      Result := reloadDefinitions;
    end;
end;

procedure TLocalFields.Notify(Item: TCollectionItem; Action: TCollectionNotification);
var
  i: Integer;
begin
  inherited;
  if Assigned(fIndex) then
    if (Action = cnDeleting) or (Action = cnExtracting) then
      if fIndex.Find(TLocalFIeld(Item).FieldIEN, i) then
        fIndex.Delete(i);
end;

procedure TLocalFields.PrepareRawData(PtData: TStrings);
var
  i, n: Integer;
begin
  n := Count - 1;
  for i:=0 to n do
    if Items[i].Toggle then
      PtData.Add(Items[i].RawData);
end;

function TLocalFields.reloadDefinitions: TReturnCode;
var
  i: Integer;
  rawData: TStringList;
begin
  Result := RC_OK;
  if RegIEN <> '' then
    begin
      Clear;
      rawData := TStringList.Create;
      try
        if Broker.CallProc(rpcListLocalFields, [RegIEN], nil, rawData) then
          begin
            for i:=1 to rawData.Count - 1 do
              AddDefinition(rawData[i]);
          end
        else
          Result := RCE_RPC_ERROR;
      finally
        rawData.Free;
      end;
    end;
end;

procedure TLocalFields.setModified(const aValue: Boolean);
var
  i: Integer;
begin
  if aValue <> fModified then
    begin
      if not aValue then
        for i:=Count-1 downto 0 do
          Items[i].Modified := False;
      fModified := aValue;
    end;
end;

procedure TLocalFields.Update(Item: TCollectionItem);
begin
  inherited;
  if Assigned(Item) and TLocalField(Item).Modified then
    Modified := True;
end;

///////////////////////////////// TCodeSetItem \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

procedure TCodeSetItem.AssignRawData(RawData: String);
begin
  fText := Piece(RawData, U, 2);
  fCode := Piece(RawData, U, 3);
end;

///////////////////////////////// TSetOfCodes \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

constructor TSetOfCodes.Create(const aType: String);
begin
  inherited Create(TCodeSetItem);
  fType := aType;
end;

function TSetOfCodes.Add(RawData: String): TCodeSetItem;
begin
  Result := TCodeSetItem(inherited Add);
  if Assigned(Result) then
    Result.AssignRawData(RawData);
end;

procedure TSetOfCodes.AssignTo(aDest: TPersistent);
var
  i, n: Integer;
begin
  if aDest is TStrings then
    begin
      TStrings(aDest).Clear;
      n := Count - 1;
      for i := 0 to n do
        TStrings(aDest).Add(Items[i].Text);
    end
  else
    inherited;
end;

function TSetOfCodes.IndexOfCode(aCode: String): Integer;
var
  i, n: Integer;
begin
  Result := -1;
  n := Count - 1;
  for i:=0 to n do
    if Items[i].Code = aCode then
      begin
        Result := i;
        Break;
      end;
end;

function TSetOfCodes.GetItem(anIndex: Integer): TCodeSetItem;
begin
  Result := TCodeSetItem(inherited GetItem(anIndex));
end;

function TSetOfCodes.Load(const aRegistryIEN: String): TReturnCode;
var
  i, n: Integer;
begin
  Broker.CreateResults;
  Clear;
  if Broker.CallProc(rpcSupportingEvidence, [aRegistryIEN, fType]) then
    begin
      n := StrToIntDef(Piece(Broker.Results[0],U), 0);
      for i := 1 to n do
        Add(Broker.Results[i]);
      Result := RC_OK;
    end
  else
    Result := RCE_RPC_ERROR;
end;

//////////////////////////////// TSelectionRule \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

procedure TSelectionRule.AssignRawData(RawData: String);
begin
  fName        := Piece(RawData, U, 3);
  fDate        := Piece(RawData, U, 4);
  fLocation    := Piece(RawData, U, 6);
  fDescription := Piece(RawData, U, 7);
end;

//////////////////////////////// TSelectionRules \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

constructor TSelectionRules.Create;
begin
  inherited Create(TSelectionRule);
end;

function TSelectionRules.Add(const RawData: String): TSelectionRule;
begin
  Result := TSelectionRule(inherited Add);
  if Assigned(Result) then
    Result.AssignRawData(RawData);
end;

procedure TSelectionRules.AssignRawData(PtData: TStrings);
var
  i, i1, n: Integer;
begin
  n := PtData.Count - 1;
  i1 := n + 1;
  for i:=0 to n do
    if Piece(PtData[i], U) = 'PSR' then
      begin
        i1 := i;
        Break;
      end;
  for i:=i1 to n do
    if Piece(PtData[i], U) = 'PSR' then
      Add(Piece(PtData[i],U,2,999))
    else
      Break;
end;

function TSelectionRules.GetItem(anIndex: Integer): TSelectionRule;
begin
  Result := TSelectionRule(inherited GetItem(anIndex));
end;

function TSelectionRules.Load(const aRegistryIEN, aPatientIEN: String): TReturnCode;
var
  i, n: Integer;
  rawData: TStringList;
begin
  Clear;
  rawData := TStringList.Create;
  try
    if Broker.CallProc('ROR PATIENT SELECTION RULES',
      [aRegistryIEN, aPatientIEN], nil, rawData) then
      begin
        n := StrToIntDef(Piece(rawData[0],U), 0);
        for i := 1 to n do
          Add(rawData[i]);
        Result := RC_OK;
      end
    else
      Result := RCE_RPC_ERROR;
  finally
    rawData.Free;
  end;
end;

end.
